--
-- Name: Shuttle
-- Description: Shuttle implementation for steerables
-- Version: 1.0
-- Author: Saracaen
-- Date: 24/07/2011
--

Shuttle = {};

-- Extended original functions --
function Shuttle:load(xmlFile)

	self.isShuttleEnabled = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.features#shuttle"), true);
	
	if self.isShuttleEnabled then
		self.setShuttleDirection = Shuttle.setShuttleDirection;
		self.shuttleDirection = 1;
	end;

end;

Steerable.load = Utils.appendedFunction(Steerable.load, Shuttle.load);

function Shuttle:readStream(streamId, connection)
	
	if self.isShuttleEnabled then
		local shuttleDirection = streamReadInt8(streamId);
		self:setShuttleDirection(shuttleDirection, true);
	end;
	
end;

Steerable.readStream = Utils.appendedFunction(Steerable.readStream, Shuttle.readStream);

function Shuttle:writeStream(streamId, connection)
	
	if self.isShuttleEnabled then
		streamWriteInt8(streamId, self.shuttleDirection);
	end;
	
end;

Steerable.writeStream = Utils.appendedFunction(Steerable.writeStream, Shuttle.writeStream);

function Shuttle:update(dt)
	
	if self.isShuttleEnabled then
		if self:getIsActiveForInput() then
			if InputBinding.hasEvent(InputBinding.CHANGE_DIRECTION) then
				self:setShuttleDirection(-self.shuttleDirection);
			end;
		end;
	end;

end;

Steerable.update = Utils.appendedFunction(Steerable.update, Shuttle.update);

function Shuttle:draw()

	if self.isShuttleEnabled then
		if self.isClient then
			if self.shuttleDirection == 1 then
				g_currentMission:addHelpButtonText(g_i18n:getText("Shuttle_forwards"), InputBinding.CHANGE_DIRECTION);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("Shuttle_backwards"), InputBinding.CHANGE_DIRECTION);
			end;	
		end;
	end;

end;

Steerable.draw = Utils.appendedFunction(Steerable.draw, Shuttle.draw);

-- Overwritten original functions --
function Shuttle.updateVehiclePhysics(self, axisForward, axisForwardIsAnalog, axisSide, axisSideIsAnalog, dt)

	local acceleration = 0;
	if self.isMotorStarted and self.motorStartTime <= self.time then
		acceleration = -axisForward;
		if math.abs(acceleration) > 0.8 then
			if self.motor.speedLevel ~= 0 then
				self.lastAcceleration = self.lastAcceleration*0.5;
			end;
			self.motor:setSpeedLevel(0, true)
		end;
		if self.motor.speedLevel ~= 0 then
			acceleration = self.motor.accelerations[self.motor.speedLevel];
		end;
	end;
	if self.fuelFillLevel == 0 then
		acceleration = 0;
	end;

	if self.isShuttleEnabled then
		acceleration = math.max(acceleration, 0) * self.shuttleDirection;
	end;
	
	acceleration = Steerable.calculateRealAcceleration(self, acceleration, dt);
   
   local inputAxisX = axisSide;
   if axisSideIsAnalog then
	   local targetRotatedTime = 0;
	   if inputAxisX < 0 then
		   -- 0 to maxRotTime
		   targetRotatedTime = math.min(-self.maxRotTime*inputAxisX, self.maxRotTime);
	   else
		   -- 0 to minRotTime
		   targetRotatedTime = math.max(self.minRotTime*inputAxisX, self.minRotTime);
	   end;
	   local maxTime = self.maxRotatedTimeSpeed*dt;
	   if math.abs(targetRotatedTime-self.rotatedTime) > maxTime then
		   if targetRotatedTime > self.rotatedTime then
			   self.rotatedTime = self.rotatedTime + maxTime;
		   else
			   self.rotatedTime = self.rotatedTime - maxTime;
		   end;
	   else
		   self.rotatedTime = targetRotatedTime;
	   end;
   else
	   local rotScale = math.min(1.0/(self.lastSpeed*self.speedRotScale+self.speedRotScaleOffset), 1);
	   if inputAxisX < 0 then
		   self.rotatedTime = math.min(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.maxRotTime);
	   elseif inputAxisX > 0 then
		   self.rotatedTime = math.max(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.minRotTime);
	   else
		   if self.autoRotateBackSpeed ~= 0 then
			   if self.rotatedTime > 0 then
				   self.rotatedTime = math.max(self.rotatedTime - dt/1000*self.autoRotateBackSpeed*rotScale, 0);
			   else
				   self.rotatedTime = math.min(self.rotatedTime + dt/1000*self.autoRotateBackSpeed*rotScale, 0);
			   end;
		   end;
	   end;
   end;

   if self.firstTimeRun then
	   WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeed, acceleration, false, self.requiredDriveMode)
   end;
   
end;

Steerable.updateVehiclePhysics = Shuttle.updateVehiclePhysics;

-- New functions --
function Shuttle:setShuttleDirection(direction, noEventSend)

	SetShuttleDirectionEvent.sendEvent(self, direction, noEventSend);
	self.shuttleDirection = direction;

end;